searchData={"items":[{"type":"module","title":"Orb","doc":"Write WebAssembly modules with Elixir.\n\nWebAssembly is a low-level language. You work with integers and floats, can perform operations on them like adding or multiplication, and then read and write those values to a block of memory. There’s no concept of a “string” or an “array”, let alone a “hash map” or “HTTP request”.\n\nThat’s where a library like Orb can help out. It takes full advantage of Elixir’s language features by becoming a compiler for WebAssembly. You can define WebAssembly modules in Elixir for “string” or “hash map”, and compose them together into a final module.\n\nThat WebAssembly module can then run in every major application environment: browsers, servers, the edge, and mobile devices like phones, tablets & laptops. This story is still being developed, but I believe like other web standards like JSON, HTML, and HTTP, that WebAssembly will become a first-class citizen on any platform. It’s Turing-complete, designed to be backwards compatible, fast, and works almost everywhere.","ref":"Orb.html"},{"type":"module","title":"Example - Orb","doc":"Let’s create a module that calculates the average of a set of numbers.\n\nWebAssembly modules can have state. Here will have two pieces of state: a total `count` and a running `tally`. These are stored as **globals**. (If you are familiar with object-oriented programming, you can think of them as instance variables).\n\nOur module will export two functions: `insert` and `calculate_mean`. These two functions will work with the `count` and `tally` globals.\n\n```elixir\ndefmodule CalculateMean do\n  use Orb\n\n  I32.global(count: 0, tally: 0)\n\n  wasm do\n    func insert(element: I32) do\n      @count = @count + 1\n      @tally = @tally + element\n    end\n\n    func calculate_mean(), I32 do\n      @tally / @count\n    end\n  end\nend\n```\n\nOne thing you’ll notice is that we must specify the type of function parameters and return values. Our `insert` function accepts a 32-bit integer, denoted using `I32`. It returns no value, while `calculate_mean` is annotated to return a 32-bit integer.\n\nWe get to write math with the intuitive `+` and `/` operators. Let’s see the same module without the magic: no math operators and without `@` conveniences for working with globals:\n\n```elixir\ndefmodule CalculateMean do\n  use Orb\n\n  I32.global(count: 0, tally: 0)\n\n  wasm do\n    func insert(element: I32) do\n      I32.add(global_get(:count), 1)\n      global_set(:count)\n      I32.add(global_get(:tally), element)\n      global_set(:tally)\n    end\n\n    func calculate_mean(), I32 do\n      I32.div_u(global_get(:tally), global_get(:count))\n    end\n  end\nend\n```\n\nThis is the exact same logic as before. In fact, this is what the first version expands to. Orb adds “sugar syntax” to make authoring WebAssembly nicer, to make it feel like writing Elixir or Ruby.","ref":"Orb.html#module-example"},{"type":"module","title":"Functions - Orb","doc":"In Elixir you define functions publicly available outside the module with `def/1`, and functions private to the module with `defp/1`. Orb follows the same suffix convention with `func/2` and `funcp/2`.\n\nConsumers of your WebAssembly module will only be able to call exported functions defined using `func/2`. Making a function public in WebAssembly is known as “exporting”.","ref":"Orb.html#module-functions"},{"type":"module","title":"Stack based - Orb","doc":"While it looks like Elixir, there are some key differences between it and programs written in Orb. The first is that state is mutable. While immutability is one of the best features of Elixir, in WebAssembly variables are mutable because raw computer memory is mutable.\n\nThe second key difference is that WebAssembly is stack based. Every function has an implicit stack of values that you can push and pop from. This paradigm allows WebAssembly runtimes to efficiently optimize for raw CPU registers whilst not being platform specific.\n\nIn Elixir when you write:\n\n```elixir\ndef example() do\n  1\n  2\n  3\nend\n```\n\nThe first two lines with `1` and `2` are inert — they have no effect — and the result from the function is the last line `3`.\n\nIn WebAssembly / Orb when you write the same sort of thing:\n\n```elixir\nwasm do\n  func example() do\n    1\n    2\n    3\n  end\nend\n```\n\nThen what’s happening is that we are pushing `1` onto the stack, then `2`, and then `3`. Now the stack has three items on it. Which will become our return value: a tuple of 3 integers. (Our function has no return type specified, so this will be an error if you attempted to compile the resulting module).\n\nSo the correct return type from this function would be a tuple of three integers:\n\n```elixir\nwasm do\n  func example(), {I32, I32, I32} do\n    1\n    2\n    3\n  end\nend\n```\n\nIf you prefer, Orb allows you to be explicit with your stack pushes with `Orb.DSL.push/1`:\n\n```elixir\nwasm do\n  func example(), {I32, I32, I32} do\n    push(1)\n    push(2)\n    push(3)\n  end\nend\n```\n\nYou can use the stack to unlock novel patterns, but for the most part Orb avoids the need to interact with it. It’s just something to keep in mind if you are used to lines of code with simple values not having any side effects.","ref":"Orb.html#module-stack-based"},{"type":"module","title":"Locals - Orb","doc":"Locals are variables that live for the lifetime of a function. They must be specified upfront with their type alongside the function’s definition, and are initialized to zero.\n\nHere we have two locals: `under?` and `over?`, both 32-bit integers. We can set their value and then read them again at the bottom of the function.\n\n```elixir\ndefmodule WithinRange do\n  use Orb\n\n  wasm do\n    func validate(num: I32), I32, under?: I32, over?: I32 do\n      under? = num   255\n\n      not (under? or over?)\n    end\n  end\nend\n```","ref":"Orb.html#module-locals"},{"type":"module","title":"Globals - Orb","doc":"Globals are like locals, but live for the duration of the entire running module’s life. Their initial type and value are specified upfront.\n\nGlobals by default are internal: nothing outside the module can see them. They can be exported to expose them to the outside world.\n\nWhen exporting a global you decide if it is `:readonly` or `:mutable`. Internal globals are mutable by default.\n\n```elixir\nI32.global(some_internal_global: 99)\n\nI32.global(:readonly, some_internal_global: 99)\n\nI32.export_global(:readonly, some_public_constant: 1001)\n\nI32.export_global(:mutable, some_public_variable: 42)\n\n# You can define multiple globals at once:\nI32.global(magic_number_a: 99, magic_number_b: 12, magic_number_c: -5)\n```\n\nYou can read or write to a global using the `@` prefix:\n\n```elixir\ndefmodule Counter do\n  use Orb\n\n  I32.global(counter: 0)\n\n  wasm do\n    func increment() do\n      @counter = @counter + 1\n    end\n  end\nend\n```","ref":"Orb.html#module-globals"},{"type":"module","title":"Memory - Orb","doc":"WebAssembly provides a buffer of memory when you need more than a handful global integers or floats. This is a contiguous array of random-access memory which you can freely read and write to.\n\n#","ref":"Orb.html#module-memory"},{"type":"module","title":"Pages - Orb","doc":"WebAssembly Memory comes in 64 KiB segments called pages. You use some multiple of these 64 KiB (64 * 1024 = 65,536 bytes) pages.\n\nBy default your module will have **no** memory, so you must specify how much memory you want upfront.\n\nHere’s an example with 16 pages (1 MiB) of memory:\n\n```elixir\ndefmodule Example do\n  use Orb\n\n  Memory.pages(16)\nend\n```\n\n#","ref":"Orb.html#module-pages"},{"type":"module","title":"Reading & writing memory - Orb","doc":"To read from memory, you can use the `Memory.load/2` function. This loads a value at the given memory address. Addresses are themselves 32-bit integers. This mean you can perform pointer arithmetic to calculate whatever address you need to access.\n\nHowever, this can prove unsafe as it’s easy to calculate the wrong address and corrupt your memory. For this reason, Orb provides higher level constructs for making working with memory pointers more pleasant, which are detailed later on.\n\n```elixir\ndefmodule Example do\n  use Orb\n\n  Memory.pages(1)\n\n  wasm do\n    func get_int32(), I32 do\n      Memory.load!(I32, 0x100)\n    end\n\n    func set_int32(value: I32) do\n      Memory.store!(I32, 0x100, value)\n    end\n  end\nend\n```\n\n#","ref":"Orb.html#module-reading-writing-memory"},{"type":"module","title":"Initializing memory with data - Orb","doc":"You can populate the initial memory of your module using `Orb.Memory.initial_data/1`. This accepts an memory offset and the string to write there.\n\n```elixir\ndefmodule MimeTypeDataExample do\n  use Orb\n\n  Memory.pages(1)\n\n  wasm do\n    Memory.initial_data(offset: 0x100, string: \"text/html\")\n    Memory.initial_data(offset: 0x200, string: \"\"\"\n       \n       \n       Hello world \n      \"\"\")\n\n    func get_mime_type(), I32 do\n      0x100\n    end\n\n    func get_body(), I32 do\n      0x200\n    end\n  end\nend\n```\n\nHaving to manually allocate and remember each memory offset is a pain, so Orb provides conveniences which are detailed in the next section.","ref":"Orb.html#module-initializing-memory-with-data"},{"type":"module","title":"Strings constants - Orb","doc":"You can use constant strings with the `~S` sigil. These will be extracted as initial data definitions at the start of the WebAssembly module, and their memory offsets substituted in their place.\n\nEach string is packed together for maximum efficiency of memory space. Strings are deduplicated, so you can use the same string constant multiple times and a single allocation will be made.\n\nString constants in Orb are nul-terminated.\n\n```elixir\ndefmodule MimeTypeStringExample do\n  use Orb\n\n  Memory.pages(1)\n\n  wasm do\n    func get_mime_type(), I32 do\n      ~S\"text/html\"\n    end\n\n    func get_body(), I32 do\n      ~S\"\"\"\n       \n       \n       Hello world \n      \"\"\"\n    end\n  end\nend\n```","ref":"Orb.html#module-strings-constants"},{"type":"module","title":"Control flow - Orb","doc":"Orb supports control flow with `if`, `block`, and `loop` statements.\n\n#","ref":"Orb.html#module-control-flow"},{"type":"module","title":"If statements - Orb","doc":"If you want to run logic conditionally, use an `if` statement.\n\n```elixir\nif @party_mode? do\n  music_volume = 100\nend\n```\n\nYou can add an `else` clause:\n\n```elixir\nif @party_mode? do\n  music_volume = 100\nelse\n  music_volume = 30\nend\n```\n\nIf you want a ternary operator (e.g. to map from one value to another), you can use `Orb.I32.when?/2` instead:\n\n```elixir\nmusic_volume = I32.when? @party_mode? do\n  100\nelse\n  30\nend\n```\n\nThese can be written on single line too:\n\n```elixir\nmusic_volume = I32.when?(@party_mode?, do: 100, else: 30)\n```\n\n#","ref":"Orb.html#module-if-statements"},{"type":"module","title":"Loops - Orb","doc":"Loops look like the familiar construct in other languages like JavaScript, with two key differences: each loop has a name, and loops by default stop unless you explicitly tell them to continue.\n\n```elixir\ni = 0\nloop CountUp do\n  i = i + 1\n\n  CountUp.continue(if: i < 10)\nend\n```\n\nEach loop is named, so if you nest them you can specify which particular one to continue.\n\n```elixir\ntotal_weeks = 10\nweekday_count = 7\nweek = 0\nweekday = 0\nloop Weeks do\n  loop Weekdays do\n    # Do something here with week and weekday\n\n    weekday = weekday + 1\n    Weekdays.continue(if: weekday < weekday_count)\n  end\n\n  week = week + 1\n  Weeks.continue(if: week < total_weeks)\nend\n```\n\n##","ref":"Orb.html#module-loops"},{"type":"module","title":"Iterators - Orb","doc":"Iterators are an upcoming feature, currently part of SilverOrb that will hopefully become part of Orb itself.\n\n#","ref":"Orb.html#module-iterators"},{"type":"module","title":"Blocks - Orb","doc":"Blocks provide a structured way to skip code.\n\n```elixir\ndefblock Validate do\n  break(Validate, if: i < 0)\n\n  # Do something with i\nend\n```\n\nBlocks can have a type.\n\n```elixir\ndefblock Double, I32 do\n  if i < 0 do\n    push(0)\n    break(Double)\n  end\n\n  push(i * 2)\nend\n```","ref":"Orb.html#module-blocks"},{"type":"module","title":"Calling other functions - Orb","doc":"You can `Orb.DSL.call/1` other functions defined within your module. Currently, the parameters and return type are not checked, so you must ensure you are calling with the correct arity and types.\n\n```elixir\nchar = call(:encode_html_char, char)\n```","ref":"Orb.html#module-calling-other-functions"},{"type":"module","title":"Composing modules together - Orb","doc":"Any functions from one module can be included into another to allow code reuse.\n\nWhen you `use Orb`, `funcp` and `func` functions are defined on your Elixir module for you. Calling these from another module will copy any functions across.\n\n```elixir\ndefmodule A do\n  use Orb\n\n  wasm do\n    func square(n: I32), I32 do\n      n * n\n    end\n  end\nend\n```\n\n```elixir\ndefmodule B do\n  use Orb\n\n  # Copies all functions defined in A as private functions into this module.\n  A.funcp()\n\n  wasm do\n    func example(n: I32), I32 do\n      call(:square, 42)\n    end\n  end\nend\n```\n\nYou can pass a name to `YourSourceModule.funcp(name)` to only copy that particular function across.\n\n```elixir\ndefmodule A do\n  use Orb\n\n  wasm do\n    func square(n: I32), I32 do\n      n * n\n    end\n\n    func double(n: I32), I32 do\n      2 * n\n    end\n  end\nend\n```\n\n```elixir\ndefmodule B do\n  use Orb\n\n  A.funcp(:square)\n\n  wasm do\n    func example(n: I32), I32 do\n      call(:square, 42)\n    end\n  end\nend\n```","ref":"Orb.html#module-composing-modules-together"},{"type":"module","title":"Importing - Orb","doc":"Your running WebAssembly module can interact with the outside world by importing globals and functions.","ref":"Orb.html#module-importing"},{"type":"module","title":"Use Elixir features - Orb","doc":"- Piping\n- Module attributes\n- Inline for\n\n#","ref":"Orb.html#module-use-elixir-features"},{"type":"module","title":"Custom types with `Access` - Orb","doc":"TODO: extract this into its own section.","ref":"Orb.html#module-custom-types-with-access"},{"type":"module","title":"Define your own functions and macros - Orb","doc":"","ref":"Orb.html#module-define-your-own-functions-and-macros"},{"type":"module","title":"Hex packages - Orb","doc":"- GoldenOrb\n    - String builder\n- SilverOrb","ref":"Orb.html#module-hex-packages"},{"type":"module","title":"Running your module - Orb","doc":"","ref":"Orb.html#module-running-your-module"},{"type":"macro","title":"Orb.functype/2","doc":"","ref":"Orb.html#functype/2"},{"type":"macro","title":"Orb.snippet/3","doc":"Declare a snippet of Orb AST for reuse. Enables DSL, with additions from `mode`.","ref":"Orb.html#snippet/3"},{"type":"function","title":"Orb.to_wat/1","doc":"Convert Orb AST into WebAssembly text format.","ref":"Orb.html#to_wat/1"},{"type":"macro","title":"Orb.types/1","doc":"","ref":"Orb.html#types/1"},{"type":"macro","title":"Orb.wasm/2","doc":"Enter WebAssembly.","ref":"Orb.html#wasm/2"},{"type":"macro","title":"Orb.wasm_import/2","doc":"Declare a WebAssembly import for a function or global.","ref":"Orb.html#wasm_import/2"},{"type":"behaviour","title":"Orb.CustomType","doc":"","ref":"Orb.CustomType.html"},{"type":"callback","title":"Orb.CustomType.byte_count/0","doc":"","ref":"Orb.CustomType.html#c:byte_count/0"},{"type":"callback","title":"Orb.CustomType.load_instruction/0","doc":"","ref":"Orb.CustomType.html#c:load_instruction/0"},{"type":"callback","title":"Orb.CustomType.wasm_type/0","doc":"","ref":"Orb.CustomType.html#c:wasm_type/0"},{"type":"module","title":"Orb.DSL","doc":"The main DSL which is imported automatically when you call `Orb.wasm/2`.","ref":"Orb.DSL.html"},{"type":"function","title":"Orb.DSL.assert!/1","doc":"Asserts a condition that _must_ be true, otherwise traps.","ref":"Orb.DSL.html#assert!/1"},{"type":"function","title":"Orb.DSL.break/1","doc":"Break from a block.","ref":"Orb.DSL.html#break/1"},{"type":"function","title":"Orb.DSL.break/2","doc":"Break from a block if a condition is true.","ref":"Orb.DSL.html#break/2"},{"type":"function","title":"Orb.DSL.call/1","doc":"Call local function `f`.","ref":"Orb.DSL.html#call/1"},{"type":"function","title":"Orb.DSL.call/2","doc":"Call local function `f`, passing arguments `args` when list, or single argument otherwise.","ref":"Orb.DSL.html#call/2"},{"type":"function","title":"Orb.DSL.call/3","doc":"Call local function `f`, passing arguments `a` & `b`.","ref":"Orb.DSL.html#call/3"},{"type":"function","title":"Orb.DSL.call/4","doc":"Call local function `f`, passing argument `a`, `b`, `c`.","ref":"Orb.DSL.html#call/4"},{"type":"function","title":"Orb.DSL.call/5","doc":"Call local function `f`, passing argument `a`, `b`, `c`, `d`.","ref":"Orb.DSL.html#call/5"},{"type":"function","title":"Orb.DSL.call/6","doc":"Call local function `f`, passing argument `a`, `b`, `c`, `d`, `e`.","ref":"Orb.DSL.html#call/6"},{"type":"function","title":"Orb.DSL.call/7","doc":"Call local function `f`, passing argument `a`, `b`, `c`, `d`, `e`, `f`.","ref":"Orb.DSL.html#call/7"},{"type":"function","title":"Orb.DSL.const/1","doc":"Declare a constant string, which will be extracted to the top of the module, and its address substituted in place.","ref":"Orb.DSL.html#const/1"},{"type":"macro","title":"Orb.DSL.defblock/3","doc":"Declare a block, useful for structured control flow.","ref":"Orb.DSL.html#defblock/3"},{"type":"function","title":"Orb.DSL.do_snippet/2","doc":"","ref":"Orb.DSL.html#do_snippet/2"},{"type":"function","title":"Orb.DSL.drop/0","doc":"Drop the last pushed value from the stack.","ref":"Orb.DSL.html#drop/0"},{"type":"function","title":"Orb.DSL.drop/1","doc":"Execute the passed expression, but ignore its result by immediately dropping its value.","ref":"Orb.DSL.html#drop/1"},{"type":"function","title":"Orb.DSL.expand_type/2","doc":"","ref":"Orb.DSL.html#expand_type/2"},{"type":"function","title":"Orb.DSL.export/2","doc":"","ref":"Orb.DSL.html#export/2"},{"type":"macro","title":"Orb.DSL.func/2","doc":"","ref":"Orb.DSL.html#func/2"},{"type":"macro","title":"Orb.DSL.func/3","doc":"","ref":"Orb.DSL.html#func/3"},{"type":"macro","title":"Orb.DSL.func/4","doc":"","ref":"Orb.DSL.html#func/4"},{"type":"function","title":"Orb.DSL.funcp/1","doc":"","ref":"Orb.DSL.html#funcp/1"},{"type":"macro","title":"Orb.DSL.funcp/2","doc":"","ref":"Orb.DSL.html#funcp/2"},{"type":"macro","title":"Orb.DSL.funcp/3","doc":"","ref":"Orb.DSL.html#funcp/3"},{"type":"macro","title":"Orb.DSL.funcp/4","doc":"","ref":"Orb.DSL.html#funcp/4"},{"type":"function","title":"Orb.DSL.global_get/1","doc":"","ref":"Orb.DSL.html#global_get/1"},{"type":"function","title":"Orb.DSL.global_set/1","doc":"","ref":"Orb.DSL.html#global_set/1"},{"type":"function","title":"Orb.DSL.i32/1","doc":"","ref":"Orb.DSL.html#i32/1"},{"type":"function","title":"Orb.DSL.i32_boolean/1","doc":"","ref":"Orb.DSL.html#i32_boolean/1"},{"type":"function","title":"Orb.DSL.i32_const/1","doc":"","ref":"Orb.DSL.html#i32_const/1"},{"type":"macro","title":"Orb.DSL.inline/1","doc":"Run code at compile-time.","ref":"Orb.DSL.html#inline/1"},{"type":"macro","title":"Orb.DSL.inline/2","doc":"Run Elixir for-comprehension at compile-time.","ref":"Orb.DSL.html#inline/2"},{"type":"function","title":"Orb.DSL.local/2","doc":"","ref":"Orb.DSL.html#local/2"},{"type":"function","title":"Orb.DSL.local_get/1","doc":"","ref":"Orb.DSL.html#local_get/1"},{"type":"function","title":"Orb.DSL.local_set/1","doc":"","ref":"Orb.DSL.html#local_set/1"},{"type":"function","title":"Orb.DSL.local_tee/1","doc":"","ref":"Orb.DSL.html#local_tee/1"},{"type":"function","title":"Orb.DSL.local_tee/2","doc":"","ref":"Orb.DSL.html#local_tee/2"},{"type":"macro","title":"Orb.DSL.loop/2","doc":"Declare a loop that iterates through a source.","ref":"Orb.DSL.html#loop/2"},{"type":"macro","title":"Orb.DSL.loop/3","doc":"Declare a loop.","ref":"Orb.DSL.html#loop/3"},{"type":"function","title":"Orb.DSL.mut!/1","doc":"","ref":"Orb.DSL.html#mut!/1"},{"type":"function","title":"Orb.DSL.nop/0","doc":"A no-op instruction.\n\nSee https://en.wikipedia.org/wiki/NOP_(code)","ref":"Orb.DSL.html#nop/0"},{"type":"function","title":"Orb.DSL.push/1","doc":"Pushes a value onto the current stack.","ref":"Orb.DSL.html#push/1"},{"type":"function","title":"Orb.DSL.push/2","doc":"Push value then run the block. Useful for when you mutate a variable but want its previous value.","ref":"Orb.DSL.html#push/2"},{"type":"function","title":"Orb.DSL.raw_wat/1","doc":"For when there’s a language feature of WebAssembly that Orb doesn’t provide. Please file an issue if there’s something you wish existed. https://github.com/RoyalIcing/Orb/issues","ref":"Orb.DSL.html#raw_wat/1"},{"type":"function","title":"Orb.DSL.return/0","doc":"Return from a function. You may wish to `push/1` values before returning.","ref":"Orb.DSL.html#return/0"},{"type":"function","title":"Orb.DSL.return/1","doc":"Return from a function if a condition is true.","ref":"Orb.DSL.html#return/1"},{"type":"function","title":"Orb.DSL.return/2","doc":"Return from a function with the provided value only if a condition is true.","ref":"Orb.DSL.html#return/2"},{"type":"function","title":"Orb.DSL.typed_call/3","doc":"","ref":"Orb.DSL.html#typed_call/3"},{"type":"function","title":"Orb.DSL.unreachable!/0","doc":"Denote a point in code that should not be reachable. Traps.\n\nUseful for exhaustive conditionals or code you know will not execute.","ref":"Orb.DSL.html#unreachable!/0"},{"type":"module","title":"Orb.DefDSL","doc":"","ref":"Orb.DefDSL.html"},{"type":"macro","title":"Orb.DefDSL.defw/2","doc":"","ref":"Orb.DefDSL.html#defw/2"},{"type":"macro","title":"Orb.DefDSL.defw/3","doc":"","ref":"Orb.DefDSL.html#defw/3"},{"type":"macro","title":"Orb.DefDSL.defw/4","doc":"","ref":"Orb.DefDSL.html#defw/4"},{"type":"macro","title":"Orb.DefDSL.defwp/2","doc":"","ref":"Orb.DefDSL.html#defwp/2"},{"type":"macro","title":"Orb.DefDSL.defwp/3","doc":"","ref":"Orb.DefDSL.html#defwp/3"},{"type":"macro","title":"Orb.DefDSL.defwp/4","doc":"","ref":"Orb.DefDSL.html#defwp/4"},{"type":"module","title":"Orb.F32.DSL","doc":"32-bit float operators.","ref":"Orb.F32.DSL.html"},{"type":"function","title":"Orb.F32.DSL.*/2","doc":"","ref":"Orb.F32.DSL.html#*/2"},{"type":"function","title":"Orb.F32.DSL.+/2","doc":"","ref":"Orb.F32.DSL.html#+/2"},{"type":"function","title":"Orb.F32.DSL.-/2","doc":"","ref":"Orb.F32.DSL.html#-/2"},{"type":"function","title":"Orb.F32.DSL.//2","doc":"","ref":"Orb.F32.DSL.html#//2"},{"type":"function","title":"Orb.F32.DSL.!=/2","doc":"","ref":"Orb.F32.DSL.html#!=/2"},{"type":"function","title":"Orb.F32.DSL.!==/2","doc":"","ref":"Orb.F32.DSL.html#!==/2"},{"type":"function","title":"Orb.F32.DSL.</2","doc":"","ref":"Orb.F32.DSL.html#%3C/2"},{"type":"function","title":"Orb.F32.DSL.<=/2","doc":"","ref":"Orb.F32.DSL.html#%3C=/2"},{"type":"function","title":"Orb.F32.DSL.==/2","doc":"","ref":"Orb.F32.DSL.html#==/2"},{"type":"function","title":"Orb.F32.DSL.===/2","doc":"","ref":"Orb.F32.DSL.html#===/2"},{"type":"function","title":"Orb.F32.DSL.>/2","doc":"","ref":"Orb.F32.DSL.html#%3E/2"},{"type":"function","title":"Orb.F32.DSL.>=/2","doc":"","ref":"Orb.F32.DSL.html#%3E=/2"},{"type":"module","title":"Orb.Global.DSL","doc":"Adds @global_name support","ref":"Orb.Global.DSL.html"},{"type":"macro","title":"Orb.Global.DSL.@/1","doc":"","ref":"Orb.Global.DSL.html#@/1"},{"type":"module","title":"Orb.I32","doc":"Type for 32-bit integer.","ref":"Orb.I32.html"},{"type":"function","title":"Orb.I32.add/2","doc":"","ref":"Orb.I32.html#add/2"},{"type":"macro","title":"Orb.I32.attr_writer/1","doc":"","ref":"Orb.I32.html#attr_writer/1"},{"type":"macro","title":"Orb.I32.attr_writer/2","doc":"","ref":"Orb.I32.html#attr_writer/2"},{"type":"function","title":"Orb.I32.band/2","doc":"","ref":"Orb.I32.html#band/2"},{"type":"function","title":"Orb.I32.clz/1","doc":"","ref":"Orb.I32.html#clz/1"},{"type":"macro","title":"Orb.I32.cond/1","doc":"","ref":"Orb.I32.html#cond/1"},{"type":"function","title":"Orb.I32.ctz/1","doc":"","ref":"Orb.I32.html#ctz/1"},{"type":"function","title":"Orb.I32.div_s/2","doc":"","ref":"Orb.I32.html#div_s/2"},{"type":"function","title":"Orb.I32.div_u/2","doc":"","ref":"Orb.I32.html#div_u/2"},{"type":"macro","title":"Orb.I32.enum/2","doc":"","ref":"Orb.I32.html#enum/2"},{"type":"function","title":"Orb.I32.eq/2","doc":"","ref":"Orb.I32.html#eq/2"},{"type":"function","title":"Orb.I32.eqz/1","doc":"","ref":"Orb.I32.html#eqz/1"},{"type":"macro","title":"Orb.I32.export_enum/2","doc":"","ref":"Orb.I32.html#export_enum/2"},{"type":"macro","title":"Orb.I32.export_global/2","doc":"","ref":"Orb.I32.html#export_global/2"},{"type":"function","title":"Orb.I32.from_4_byte_ascii/1","doc":"Converts an ASCII little-endian 4-byte string to an 32-bit integer.","ref":"Orb.I32.html#from_4_byte_ascii/1"},{"type":"function","title":"Orb.I32.ge_s/2","doc":"","ref":"Orb.I32.html#ge_s/2"},{"type":"function","title":"Orb.I32.ge_u/2","doc":"","ref":"Orb.I32.html#ge_u/2"},{"type":"macro","title":"Orb.I32.global/2","doc":"","ref":"Orb.I32.html#global/2"},{"type":"function","title":"Orb.I32.gt_s/2","doc":"","ref":"Orb.I32.html#gt_s/2"},{"type":"function","title":"Orb.I32.gt_u/2","doc":"","ref":"Orb.I32.html#gt_u/2"},{"type":"function","title":"Orb.I32.in?/2","doc":"","ref":"Orb.I32.html#in?/2"},{"type":"function","title":"Orb.I32.le_s/2","doc":"","ref":"Orb.I32.html#le_s/2"},{"type":"function","title":"Orb.I32.le_u/2","doc":"","ref":"Orb.I32.html#le_u/2"},{"type":"function","title":"Orb.I32.lt_s/2","doc":"","ref":"Orb.I32.html#lt_s/2"},{"type":"function","title":"Orb.I32.lt_u/2","doc":"","ref":"Orb.I32.html#lt_u/2"},{"type":"macro","title":"Orb.I32.match/2","doc":"","ref":"Orb.I32.html#match/2"},{"type":"function","title":"Orb.I32.mul/2","doc":"","ref":"Orb.I32.html#mul/2"},{"type":"function","title":"Orb.I32.ne/2","doc":"","ref":"Orb.I32.html#ne/2"},{"type":"function","title":"Orb.I32.or/2","doc":"","ref":"Orb.I32.html#or/2"},{"type":"function","title":"Orb.I32.popcnt/1","doc":"","ref":"Orb.I32.html#popcnt/1"},{"type":"function","title":"Orb.I32.rem_s/2","doc":"","ref":"Orb.I32.html#rem_s/2"},{"type":"function","title":"Orb.I32.rem_u/2","doc":"","ref":"Orb.I32.html#rem_u/2"},{"type":"function","title":"Orb.I32.rotl/2","doc":"","ref":"Orb.I32.html#rotl/2"},{"type":"function","title":"Orb.I32.rotr/2","doc":"","ref":"Orb.I32.html#rotr/2"},{"type":"function","title":"Orb.I32.select/2","doc":"","ref":"Orb.I32.html#select/2"},{"type":"function","title":"Orb.I32.shl/2","doc":"","ref":"Orb.I32.html#shl/2"},{"type":"function","title":"Orb.I32.shr_s/2","doc":"","ref":"Orb.I32.html#shr_s/2"},{"type":"function","title":"Orb.I32.shr_u/2","doc":"","ref":"Orb.I32.html#shr_u/2"},{"type":"function","title":"Orb.I32.sub/2","doc":"","ref":"Orb.I32.html#sub/2"},{"type":"function","title":"Orb.I32.sum!/1","doc":"","ref":"Orb.I32.html#sum!/1"},{"type":"function","title":"Orb.I32.trunc_f32_s/1","doc":"","ref":"Orb.I32.html#trunc_f32_s/1"},{"type":"function","title":"Orb.I32.trunc_f32_u/1","doc":"","ref":"Orb.I32.html#trunc_f32_u/1"},{"type":"function","title":"Orb.I32.trunc_f64_s/1","doc":"","ref":"Orb.I32.html#trunc_f64_s/1"},{"type":"function","title":"Orb.I32.trunc_f64_u/1","doc":"","ref":"Orb.I32.html#trunc_f64_u/1"},{"type":"function","title":"Orb.I32.wasm_type/0","doc":"","ref":"Orb.I32.html#wasm_type/0"},{"type":"macro","title":"Orb.I32.when?/2","doc":"","ref":"Orb.I32.html#when?/2"},{"type":"function","title":"Orb.I32.xor/2","doc":"","ref":"Orb.I32.html#xor/2"},{"type":"module","title":"Orb.I32.DSL","doc":"The operators that work consistently signed or unsigned.\n\nSigned specific: `Orb.S32.DSL`\n\nUnsigned specific: `Orb.U32.DSL`","ref":"Orb.I32.DSL.html"},{"type":"function","title":"Orb.I32.DSL.&&&/2","doc":"","ref":"Orb.I32.DSL.html#&&&/2"},{"type":"function","title":"Orb.I32.DSL.*/2","doc":"","ref":"Orb.I32.DSL.html#*/2"},{"type":"function","title":"Orb.I32.DSL.+/2","doc":"","ref":"Orb.I32.DSL.html#+/2"},{"type":"function","title":"Orb.I32.DSL.-/2","doc":"","ref":"Orb.I32.DSL.html#-/2"},{"type":"function","title":"Orb.I32.DSL.!=/2","doc":"","ref":"Orb.I32.DSL.html#!=/2"},{"type":"function","title":"Orb.I32.DSL.!==/2","doc":"","ref":"Orb.I32.DSL.html#!==/2"},{"type":"function","title":"Orb.I32.DSL.<<</2","doc":"","ref":"Orb.I32.DSL.html#%3C%3C%3C/2"},{"type":"function","title":"Orb.I32.DSL.==/2","doc":"","ref":"Orb.I32.DSL.html#==/2"},{"type":"function","title":"Orb.I32.DSL.===/2","doc":"","ref":"Orb.I32.DSL.html#===/2"},{"type":"function","title":"Orb.I32.DSL.not/1","doc":"","ref":"Orb.I32.DSL.html#not/1"},{"type":"function","title":"Orb.I32.DSL.or/2","doc":"","ref":"Orb.I32.DSL.html#or/2"},{"type":"function","title":"Orb.I32.DSL.|||/2","doc":"","ref":"Orb.I32.DSL.html#%7C%7C%7C/2"},{"type":"module","title":"Orb.I32.String","doc":"Custom type for a nul-terminated ASCII string.","ref":"Orb.I32.String.html"},{"type":"function","title":"Orb.I32.String.funcp/0","doc":"Import all WebAssembly functions from this module’s Orb definition.","ref":"Orb.I32.String.html#funcp/0"},{"type":"function","title":"Orb.I32.String.funcp/1","doc":"Import a specific WebAssembly function from this module’s Orb definition.","ref":"Orb.I32.String.html#funcp/1"},{"type":"macro","title":"Orb.I32.String.match/2","doc":"","ref":"Orb.I32.String.html#match/2"},{"type":"function","title":"Orb.I32.String.null/0","doc":"","ref":"Orb.I32.String.html#null/0"},{"type":"function","title":"Orb.I32.String.streq/2","doc":"","ref":"Orb.I32.String.html#streq/2"},{"type":"function","title":"Orb.I32.String.strlen/1","doc":"","ref":"Orb.I32.String.html#strlen/1"},{"type":"function","title":"Orb.I32.String.to_wat/0","doc":"Convert this module’s Orb definition to WebAssembly text (Wat) format.","ref":"Orb.I32.String.html#to_wat/0"},{"type":"module","title":"Orb.I32.String.CharIterator","doc":"Custom `Orb.CustomType`. Iterate over each byte character in an ASCII string.","ref":"Orb.I32.String.CharIterator.html"},{"type":"module","title":"Orb.I32.U8","doc":"Type for unsigned 32-bit integer interpreted as an 8-bit byte.","ref":"Orb.I32.U8.html"},{"type":"module","title":"Orb.I32.U8.UnsafePointer","doc":"Custom `Orb.CustomType` for pointer to 8-bit integer (byte) in memory.","ref":"Orb.I32.U8.UnsafePointer.html"},{"type":"module","title":"Orb.I32.UnsafePointer","doc":"Custom `Orb.CustomType` for pointer to 32-bit integer in memory.","ref":"Orb.I32.UnsafePointer.html"},{"type":"module","title":"Orb.I64","doc":"Type for 64-bit integer.","ref":"Orb.I64.html"},{"type":"module","title":"Orb.Instruction","doc":"","ref":"Orb.Instruction.html"},{"type":"function","title":"Orb.Instruction.call/1","doc":"","ref":"Orb.Instruction.html#call/1"},{"type":"function","title":"Orb.Instruction.call/2","doc":"","ref":"Orb.Instruction.html#call/2"},{"type":"function","title":"Orb.Instruction.f32/1","doc":"","ref":"Orb.Instruction.html#f32/1"},{"type":"function","title":"Orb.Instruction.f32/2","doc":"","ref":"Orb.Instruction.html#f32/2"},{"type":"function","title":"Orb.Instruction.f32/3","doc":"","ref":"Orb.Instruction.html#f32/3"},{"type":"function","title":"Orb.Instruction.i32/1","doc":"","ref":"Orb.Instruction.html#i32/1"},{"type":"function","title":"Orb.Instruction.i32/2","doc":"","ref":"Orb.Instruction.html#i32/2"},{"type":"function","title":"Orb.Instruction.i32/3","doc":"","ref":"Orb.Instruction.html#i32/3"},{"type":"function","title":"Orb.Instruction.local_get/2","doc":"","ref":"Orb.Instruction.html#local_get/2"},{"type":"function","title":"Orb.Instruction.new/2","doc":"","ref":"Orb.Instruction.html#new/2"},{"type":"function","title":"Orb.Instruction.new/3","doc":"","ref":"Orb.Instruction.html#new/3"},{"type":"function","title":"Orb.Instruction.new/4","doc":"","ref":"Orb.Instruction.html#new/4"},{"type":"function","title":"Orb.Instruction.typed_call/3","doc":"","ref":"Orb.Instruction.html#typed_call/3"},{"type":"module","title":"Orb.Memory","doc":"Work with memory.","ref":"Orb.Memory.html"},{"type":"function","title":"Orb.Memory.initial_data/1","doc":"Initializes data in memory. In Wat is `(data …)`","ref":"Orb.Memory.html#initial_data/1"},{"type":"function","title":"Orb.Memory.initial_data_prepacked/1","doc":"Initializes data in memory from a packed `Map`. In Wat is `(data …)`","ref":"Orb.Memory.html#initial_data_prepacked/1"},{"type":"function","title":"Orb.Memory.load!/2","doc":"Load value of `type` from memory `address`.\n\n```elixir\nMemory.load!(I32, 0x100)\n```","ref":"Orb.Memory.html#load!/2"},{"type":"macro","title":"Orb.Memory.pages/1","doc":"Declare how many 64Kib pages of memory your module needs.\n\nCan be called multiple times: the highest value is used.","ref":"Orb.Memory.html#pages/1"},{"type":"function","title":"Orb.Memory.store!/3","doc":"Store `value` of `type` at memory `address`.\n\n```elixir\nMemory.store!(I32, 0x100, 42)\n```","ref":"Orb.Memory.html#store!/3"},{"type":"module","title":"Orb.MutRef","doc":"Use `Orb.DSL.mut!/1` to get a mutable reference to a global or local.","ref":"Orb.MutRef.html"},{"type":"function","title":"Orb.MutRef.from/1","doc":"","ref":"Orb.MutRef.html#from/1"},{"type":"function","title":"Orb.MutRef.store/2","doc":"","ref":"Orb.MutRef.html#store/2"},{"type":"module","title":"Orb.Numeric.Add","doc":"WebAssembly numeric `add` instruction:\n- `i32.add`\n- `i64.add`\n- `f32.add`\n- `f64.add`\n\n[MDN reference](https://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Numeric/Addition)","ref":"Orb.Numeric.Add.html"},{"type":"function","title":"Orb.Numeric.Add.optimized/3","doc":"","ref":"Orb.Numeric.Add.html#optimized/3"},{"type":"module","title":"Orb.Numeric.Multiply","doc":"WebAssembly numeric `mul` instruction:\n- `i32.mul`\n- `i64.mul`\n- `f32.mul`\n- `f64.mul`\n\n[MDN reference](https://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Numeric/Multiplication)","ref":"Orb.Numeric.Multiply.html"},{"type":"function","title":"Orb.Numeric.Multiply.optimized/3","doc":"","ref":"Orb.Numeric.Multiply.html#optimized/3"},{"type":"module","title":"Orb.Numeric.NotEqual","doc":"WebAssembly numeric `ne` instruction:\n- `i32.ne`\n- `i64.ne`\n- `f32.ne`\n- `f64.ne`\n\n[MDN reference](https://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Numeric/Not_equal)","ref":"Orb.Numeric.NotEqual.html"},{"type":"function","title":"Orb.Numeric.NotEqual.optimized/3","doc":"","ref":"Orb.Numeric.NotEqual.html#optimized/3"},{"type":"module","title":"Orb.Numeric.Subtract","doc":"WebAssembly numeric `sub` instruction:\n- `i32.sub`\n- `i64.sub`\n- `f32.sub`\n- `f64.sub`\n\n[MDN reference](https://developer.mozilla.org/en-US/docs/WebAssembly/Reference/Numeric/Subtraction)","ref":"Orb.Numeric.Subtract.html"},{"type":"function","title":"Orb.Numeric.Subtract.optimized/3","doc":"","ref":"Orb.Numeric.Subtract.html#optimized/3"},{"type":"module","title":"Orb.S32.DSL","doc":"Signed 32-bit integer operators.","ref":"Orb.S32.DSL.html"},{"type":"function","title":"Orb.S32.DSL.//2","doc":"","ref":"Orb.S32.DSL.html#//2"},{"type":"function","title":"Orb.S32.DSL.</2","doc":"","ref":"Orb.S32.DSL.html#%3C/2"},{"type":"function","title":"Orb.S32.DSL.<=/2","doc":"","ref":"Orb.S32.DSL.html#%3C=/2"},{"type":"function","title":"Orb.S32.DSL.>/2","doc":"","ref":"Orb.S32.DSL.html#%3E/2"},{"type":"function","title":"Orb.S32.DSL.>=/2","doc":"","ref":"Orb.S32.DSL.html#%3E=/2"},{"type":"function","title":"Orb.S32.DSL.>>>/2","doc":"","ref":"Orb.S32.DSL.html#%3E%3E%3E/2"},{"type":"module","title":"Orb.Table","doc":"","ref":"Orb.Table.html"},{"type":"macro","title":"Orb.Table.allocate/1","doc":"","ref":"Orb.Table.html#allocate/1"},{"type":"function","title":"Orb.Table.call/1","doc":"","ref":"Orb.Table.html#call/1"},{"type":"function","title":"Orb.Table.call/2","doc":"","ref":"Orb.Table.html#call/2"},{"type":"function","title":"Orb.Table.call/3","doc":"","ref":"Orb.Table.html#call/3"},{"type":"function","title":"Orb.Table.call/4","doc":"","ref":"Orb.Table.html#call/4"},{"type":"function","title":"Orb.Table.call/5","doc":"","ref":"Orb.Table.html#call/5"},{"type":"function","title":"Orb.Table.call_indirect/2","doc":"","ref":"Orb.Table.html#call_indirect/2"},{"type":"function","title":"Orb.Table.call_indirect/3","doc":"","ref":"Orb.Table.html#call_indirect/3"},{"type":"function","title":"Orb.Table.call_indirect/4","doc":"","ref":"Orb.Table.html#call_indirect/4"},{"type":"function","title":"Orb.Table.call_indirect/5","doc":"","ref":"Orb.Table.html#call_indirect/5"},{"type":"function","title":"Orb.Table.call_indirect/6","doc":"","ref":"Orb.Table.html#call_indirect/6"},{"type":"function","title":"Orb.Table.do_elem/4","doc":"","ref":"Orb.Table.html#do_elem/4"},{"type":"macro","title":"Orb.Table.elem!/3","doc":"","ref":"Orb.Table.html#elem!/3"},{"type":"macro","title":"Orb.Table.fetch!/1","doc":"","ref":"Orb.Table.html#fetch!/1"},{"type":"macro","title":"Orb.Table.lookup!/1","doc":"","ref":"Orb.Table.html#lookup!/1"},{"type":"macro","title":"Orb.Table.lookup!/2","doc":"","ref":"Orb.Table.html#lookup!/2"},{"type":"module","title":"Orb.Table.CallIndirect","doc":"","ref":"Orb.Table.CallIndirect.html"},{"type":"protocol","title":"Orb.ToWat","doc":"","ref":"Orb.ToWat.html"},{"type":"function","title":"Orb.ToWat.to_wat/2","doc":"Protocol for converting to WebAssembly text format (Wat)","ref":"Orb.ToWat.html#to_wat/2"},{"type":"type","title":"Orb.ToWat.t/0","doc":"All the types that implement this protocol.","ref":"Orb.ToWat.html#t:t/0"},{"type":"module","title":"Orb.Type","doc":"","ref":"Orb.Type.html"},{"type":"module","title":"Orb.U32.DSL","doc":"Unsigned 32-bit integer operators.\n\nSome operators like division have two variations in WebAssembly: signed and unsigned. By default math perform signed operations, but if you want unsigned math you can pass `U32` to `Orb.wasm/2` like so: `wasm U32 do`.","ref":"Orb.U32.DSL.html"},{"type":"function","title":"Orb.U32.DSL.//2","doc":"","ref":"Orb.U32.DSL.html#//2"},{"type":"function","title":"Orb.U32.DSL.</2","doc":"","ref":"Orb.U32.DSL.html#%3C/2"},{"type":"function","title":"Orb.U32.DSL.<=/2","doc":"","ref":"Orb.U32.DSL.html#%3C=/2"},{"type":"function","title":"Orb.U32.DSL.>/2","doc":"","ref":"Orb.U32.DSL.html#%3E/2"},{"type":"function","title":"Orb.U32.DSL.>=/2","doc":"","ref":"Orb.U32.DSL.html#%3E=/2"},{"type":"function","title":"Orb.U32.DSL.>>>/2","doc":"","ref":"Orb.U32.DSL.html#%3E%3E%3E/2"},{"type":"exception","title":"Orb.ModuleDefinition.FetchFuncError","doc":"","ref":"Orb.ModuleDefinition.FetchFuncError.html"},{"type":"exception","title":"Orb.TypeCheckError","doc":"","ref":"Orb.TypeCheckError.html"},{"type":"extras","title":"README","doc":"![Orb logo](orb-logo-orange.svg)\n\n# Orb: Write WebAssembly with the power of Elixir\n\nOrb is a DSL for WebAssembly with the full power of the Elixir language:\n\n- Function piping, Macros\n- Break problems into smaller modules and compose them together\n- Share code via the Hex package manager","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"The package can be installed via Hex by adding `orb` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:orb, \"~> 0.0.14\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Example - README","doc":"```elixir\ndefmodule CalculateMean do\n  use Orb\n\n  I32.global(\n    count: 0,\n    tally: 0\n  )\n\n  wasm do\n    func insert(element: I32) do\n      @count = @count + 1\n      @tally = @tally + element\n    end\n\n    func calculate_mean(), I32 do\n      @tally / @count\n    end\n  end\nend\n\nOrb.to_wat(CalculateMean)\n# \"\"\"\n# (module $CalculateMean\n#   (global $count (mut i32) (i32.const 0))\n#   (global $tally (mut i32) (i32.const 0))\n#   (func $insert (export \"insert\") (param $element i32)\n#     (i32.add (global.get $count) (i32.const 1))\n#     (global.set $count)\n#     (i32.add (global.get $tally) (local.get $element))\n#     (global.set $tally)\n#   )\n#   (func $calculate_mean (export \"calculate_mean\") (result i32)\n#     (i32.div_s (global.get $tally) (global.get $count))\n#   )\n# )\n# \"\"\"\n```\n\nRun with [OrbWasmtime](https://github.com/RoyalIcing/OrbWasmtime):\n\n```elixir\nalias OrbWasmtime.Instance\n\n# Run above example\ninst = Instance.run(CalculateMean)\nInstance.call(inst, :insert, 4)\nInstance.call(inst, :insert, 5)\nInstance.call(inst, :insert, 6)\nassert Instance.call(inst, :calculate_mean) == 5\n```\n\nNote there is [another excellent Elixir Wasmtime wrapper out there called Wasmex](https://github.com/tessi/wasmex), you may want to check that out too.","ref":"readme.html#example"},{"type":"extras","title":"Use cases - README","doc":"- Write a HTML component and run it in:\n    - Phoenix LiveView & dead views\n    - In the browser using ` ` custom element\n- LiveView and its server rendering is a fantastic default, but the latency can be noticeable for certain UI interactions. With Orb you could use Elixir to write a WebAssembly module that then runs in the user’s browser.\n- State machines\n- Parsers\n- Formatters & string builders\n- Interactive color picker controls\n- Animation that runs fast in the browser, and works on the server\n- Code generators","ref":"readme.html#use-cases"},{"type":"extras","title":"Why WebAssembly? - README","doc":"- It runs on all of today’s major platforms: browser, server, edge, mobile, laptop, tablet, desktop.\n- Universal/isomorphic components (ones that run on the server and browser) are possible in React and Next.js, but they have many different flavours and can get pretty complex for a system that was meant to be declarative.\n- Like HTML and CSS it’s backwards compatible, which means WebAssembly you author today will be guaranteed to still work in a decade or more.\n- It’s memory-safe and sandboxed. It can’t read memory outside of itself, plus only what has been explicitly passed into it. It can be timeboxed to run only for a certain duration.\n- It’s fast.","ref":"readme.html#why-webassembly"},{"type":"extras","title":"Why develop Orb in Elixir? - README","doc":"Here are the reasons I chose to write Orb in Elixir.\n\n- Established language:\n    - Has package manager.\n    - Has composable modules with `alias` & `use`.\n    - Has syntax highlighting in IDEs, GitHub, and in highlighting libraries.\n    - Has language server with autocomplete.\n    - Has documentation system.\n    - Has unit test library.\n    - Has CI integration.\n    - Has linting.\n    - Integrates with native libraries in Rust and Zig.\n    - Has upcoming type system.\n- Established frameworks:\n    - Can integrate with Phoenix LiveView.\n    - Can connect to cloud, databases.\n    - Can integrate with Rust.\n- Community that is friendly.\n- Can be extended with additional functions and macros:\n  - Unlike say C’s basic string-inserting preprocessor, Elixir is a full programming language without constraints.\n  - We can read files or the network and then generate code.\n  - You can create your own DSL. Want to enforce immutable-style programming? Want to add pattern matching? Design a DSL for it.\n\n----\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#why-develop-orb-in-elixir"},{"type":"extras","title":"Orb Introduction","doc":"# Orb Introduction\n\n```elixir\nMix.install([\n  {:jason, \"~> 1.0\"},\n  :orb,\n  :orb_wasmtime\n])\n```","ref":"01-intro.html"},{"type":"extras","title":"Section - Orb Introduction","doc":"```elixir\nalias OrbWasmtime.Instance, as: Wasmtime\n```\n\n```elixir\ndefmodule Mean do\n  use Orb\n\n  I32.global(sum: 0, count: 0)\n\n  wasm do\n    func insert(n: I32) do\n      @count = @count + 1\n      @sum = @sum + n\n    end\n\n    func calculate_mean(), I32 do\n      @sum / @count\n    end\n  end\nend\n```\n\n```elixir\na = Wasmtime.run(Mean)\nWasmtime.call(a, :insert, 3)\nWasmtime.call(a, :insert, 4)\nWasmtime.call(a, :insert, 5)\nWasmtime.call(a, :insert, 6)\nWasmtime.call(a, :insert, 7)\nWasmtime.call(a, :calculate_mean)\n```","ref":"01-intro.html#section"}],"content_type":"text/markdown"}